@@ -868,52 +868,71 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 	    if (iconw)
 		gtk_widget_show (iconw);
 
-	    if (new_group)
-		gtk_toolbar_append_space (GTK_TOOLBAR (parent));
+	    if (new_group) {
+		GtkWidget *toolitem = GTK_WIDGET (gtk_tool_item_new ());
+
+		gtk_container_add (GTK_CONTAINER (parent), toolitem);
+		gtk_widget_show (toolitem);
+	    }
 
 	    /* FIXME: these should be translated */
 	    if (!strcmp (childinfo->child->classname, "toggle")) {
-		child = gtk_toolbar_append_element (
-		    GTK_TOOLBAR (parent),
-		    GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL,
-		    label, tooltip, NULL, iconw, NULL, NULL);
-		gtk_toggle_button_set_active
-		    (GTK_TOGGLE_BUTTON (child), active);
+		child = g_object_new (GTK_TYPE_TOGGLE_TOOL_BUTTON,
+				      "label", label,
+				      "stock_id", stock,
+				      NULL);
+		gtk_toggle_tool_button_set_active
+		    (GTK_TOGGLE_TOOL_BUTTON (child), active);
 	    } else if (!strcmp (childinfo->child->classname, "radio")) {
-		child = gtk_toolbar_append_element (
-		    GTK_TOOLBAR (parent),
-		    GTK_TOOLBAR_CHILD_RADIOBUTTON, NULL,
-		    label, tooltip, NULL, iconw, NULL, NULL);
-
+		child = g_object_new (GTK_TYPE_RADIO_TOOL_BUTTON,
+				      "label", label,
+				      "stock_id", stock,
+				      NULL);
 		if (group_name) {
 		    g_object_set (G_OBJECT (child),
 				  "group", glade_xml_get_widget (xml, group_name),
 				  NULL);
 		}
-		gtk_toggle_button_set_active
-		    (GTK_TOGGLE_BUTTON (child), active);
-	    } else
-		child = gtk_toolbar_append_item (
-		    GTK_TOOLBAR (parent),
-		    label, tooltip, NULL, iconw, NULL, NULL);
+		gtk_toggle_tool_button_set_active
+		    (GTK_TOGGLE_TOOL_BUTTON (child), active);
+	    } else {
+		child = g_object_new (GTK_TYPE_TOOL_BUTTON,
+				      "label", label,
+				      "stock_id", stock,
+				      NULL);
+	    }
+	    if (iconw)
+		gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (child),
+						 iconw);
 	    
 	    /* GTK+ doesn't support use_underline directly, so we have to hack
 	       it. */
 	    if (use_underline) {
-		GList *elem = g_list_last (GTK_TOOLBAR (parent)->children);
-		GtkToolbarChild *toolbar_child = elem->data;
-		gtk_label_set_use_underline (GTK_LABEL (toolbar_child->label),
-					     TRUE);
+		GtkWidget *labelw = gtk_tool_button_get_label_widget (GTK_TOOL_BUTTON (child));
+		gtk_label_set_use_underline (GTK_LABEL (labelw), TRUE);
 	    }
 
+	    if (tooltip) {
+		gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (child),
+					   xml->priv->tooltips,
+					   tooltip, NULL);
+		
+	    }
+
+	    gtk_container_add (GTK_CONTAINER (parent), child);
+
 	    glade_xml_set_common_params (xml, child, childinfo->child);
 	} else {
 	    child = glade_xml_build_widget (xml, childinfo->child);
 
-	    if (GTK_IS_TOOL_ITEM (child))
-		gtk_toolbar_insert (GTK_TOOLBAR (parent), GTK_TOOL_ITEM (child), -1);
-	    else
-		gtk_toolbar_append_widget (GTK_TOOLBAR (parent), child, NULL, NULL);
+	    if (!GTK_IS_TOOL_ITEM (child)) {
+		GtkWidget *toolitem = GTK_WIDGET (gtk_tool_item_new ());
+
+		gtk_container_add (GTK_CONTAINER (toolitem), child);
+		gtk_widget_show (toolitem);
+		child = toolitem;
+	    }
+	    gtk_container_add (GTK_CONTAINER (parent), child);
 	}
     }
 }
@@ -1168,6 +1187,8 @@ _glade_init_gtk_widgets(void)
     glade_register_custom_prop (GTK_TYPE_TOOL_BUTTON, "icon", tool_button_set_icon);
     glade_register_custom_prop (GTK_TYPE_COMBO_BOX, "items", combo_box_set_items);
 
+    glade_register_widget (GTK_TYPE_ABOUT_DIALOG, NULL,
+			   NULL, NULL);
     glade_register_widget (GTK_TYPE_ACCEL_LABEL, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_ALIGNMENT, glade_standard_build_widget,
@@ -1215,7 +1236,9 @@ _glade_init_gtk_widgets(void)
     glade_register_widget (GTK_TYPE_FILE_CHOOSER, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_FILE_CHOOSER_DIALOG, glade_standard_build_widget,
-			   gtk_dialog_build_children, dialog_find_internal_child);
+			   NULL, NULL);
+    glade_register_widget (GTK_TYPE_FILE_CHOOSER_BUTTON, glade_standard_build_widget,
+			   NULL, NULL);
     glade_register_widget (GTK_TYPE_FILE_SELECTION, NULL,
 			   glade_standard_build_children, filesel_find_internal_child);
     glade_register_widget (GTK_TYPE_FIXED, glade_standard_build_widget,
@@ -1246,6 +1269,8 @@ _glade_init_gtk_widgets(void)
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_HSEPARATOR, glade_standard_build_widget,
 			   NULL, NULL);
+    glade_register_widget (GTK_TYPE_ICON_VIEW, glade_standard_build_widget,
+			   NULL, NULL);
     glade_register_widget (GTK_TYPE_IMAGE, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_IMAGE_MENU_ITEM, glade_standard_build_widget,
@@ -1266,6 +1291,8 @@ _glade_init_gtk_widgets(void)
 			   glade_standard_build_children, NULL);
     glade_register_widget (GTK_TYPE_MENU_ITEM, glade_standard_build_widget,
 			   menuitem_build_children, NULL);
+    glade_register_widget (GTK_TYPE_MENU_TOOL_BUTTON, glade_standard_build_widget,
+			   NULL, NULL);
     glade_register_widget (GTK_TYPE_MESSAGE_DIALOG, NULL,
 			   glade_standard_build_children, NULL);
     glade_register_widget (GTK_TYPE_NOTEBOOK, glade_standard_build_widget,
