@@ -277,7 +277,7 @@ autoconnect_foreach(const char *signal_handler, GList *signals,
 {
     GCallback func;
 
-    if (!g_module_symbol(allsymbols, signal_handler, (gpointer *)&func))
+    if (!g_module_symbol(allsymbols, signal_handler, (gpointer)&func))
 	g_warning("could not find signal handler '%s'.", signal_handler);
     else
 	for (; signals != NULL; signals = signals->next) {
@@ -368,8 +368,8 @@ autoconnect_full_foreach(const char *signal_handler, GList *signals,
  * @object: the object to connect the signal to.
  * @signal_name: the name of the signal.
  * @signal_data: the string value of the signal data given in the XML file.
- * @connect_object: non NULL if gtk_signal_connect_object should be used.
- * @after: TRUE if the connection should be made with gtk_signal_connect_after.
+ * @connect_object: non NULL if g_signal_connect_object should be used.
+ * @after: TRUE if the connection should be made with g_signal_connect_after.
  * @user_data: the user data argument.
  *
  * This is the signature of a function used to connect signals.  It is used
@@ -788,14 +788,22 @@ add_relation(AtkRelationSet *relations, AtkRelationType relation_type,
     if (relation) {
 	/* add new target accessible to relation */
 	GPtrArray* target_array = atk_relation_get_target (relation);
+	GPtrArray* new_target_array;
+	AtkRelation *new_relation;
 	guint i;
 
 	/* first check if target occurs in array ... */
 	for (i = 0; i < target_array->len; i++)
 	    if (g_ptr_array_index(target_array, i) == target_accessible)
 		return;
-	g_object_ref(target_accessible);
-	g_ptr_array_add (target_array, target_accessible);
+        new_target_array = g_ptr_array_sized_new (target_array->len + 1);
+	for (i = 0; i < target_array->len; i++)
+	    g_ptr_array_add (new_target_array, g_ptr_array_index (target_array, i));
+	g_ptr_array_add (new_target_array, target_accessible);
+        new_relation = atk_relation_new (&g_ptr_array_index (new_target_array, 0), new_target_array->len, relation_type);
+	atk_relation_set_remove (relations, relation);
+	atk_relation_set_add (relations, new_relation);
+        g_ptr_array_free (new_target_array, TRUE);
     } else {
 	/* the relation hasn't been created yet ... */
 	relation = atk_relation_new(&target_accessible, 1, relation_type);
@@ -1468,7 +1476,7 @@ glade_xml_set_value_from_string (GladeXML *xml,
 				 GValue *value)
 {
     GType prop_type;
-    gboolean ret = TRUE;
+    gboolean ret = TRUE, showerr = TRUE;
 
     prop_type = G_PARAM_SPEC_VALUE_TYPE(pspec);
     g_value_init(value, prop_type);
@@ -1524,6 +1532,9 @@ glade_xml_set_value_from_string (GladeXML *xml,
 		g_warning ("could not parse colour name `%s'", string);
 		ret = FALSE;
 	    }
+	} else if (G_VALUE_HOLDS(value, G_TYPE_STRV)) {
+	    char **vector = g_strsplit (string, "\n", 0);
+	    g_value_take_boxed (value, vector);
 	} else
 	    ret = FALSE;
 	break;
@@ -1558,6 +1569,16 @@ glade_xml_set_value_from_string (GladeXML *xml,
 		ret = FALSE;
 	    }
 	    g_free(filename);
+	} else if (g_type_is_a(GTK_TYPE_WIDGET, G_PARAM_SPEC_VALUE_TYPE(pspec)) ||
+		   g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(pspec), GTK_TYPE_WIDGET)) {
+	    GtkWidget *widget = g_hash_table_lookup(xml->priv->name_hash,
+						    string);
+	    if (widget) {
+		g_value_set_object(value, widget);
+	    } else {
+		ret = FALSE;
+		showerr = FALSE;
+	    }
 	} else
 	    ret = FALSE;
 	break;
@@ -1567,8 +1588,10 @@ glade_xml_set_value_from_string (GladeXML *xml,
     }
 
     if (!ret) {
-	g_warning("could not convert string to type `%s' for property `%s'",
-		  g_type_name(prop_type), pspec->name);
+	if (showerr)
+	    g_warning("could not convert string to type "
+		      "`%s' for property `%s'",
+		      g_type_name(prop_type), pspec->name);
 	g_value_unset(value);
     }
     return ret;
@@ -1647,25 +1670,17 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
 	    continue;
 	}
 
-	/* this should catch all properties wanting a GtkWidget
-         * subclass.  We also look for types that could hold a
-         * GtkWidget in order to catch things like the
-         * GtkAccelLabel::accel_object property.  Since we don't do
-         * any handling of GObject or GtkObject directly in
-         * glade_xml_set_value_from_string, this shouldn't be a
-         * problem. */
-	if (g_type_is_a(GTK_TYPE_WIDGET, G_PARAM_SPEC_VALUE_TYPE(pspec)) ||
-	    g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(pspec), GTK_TYPE_WIDGET)) {
-	    deferred_props = g_list_prepend(deferred_props,
-					    &info->properties[i]);
-	    continue;
-	}
-
 	if (glade_xml_set_value_from_string(xml, pspec,
 					    info->properties[i].value,
 					    &param.value)) {
 	    param.name = info->properties[i].name;
 	    g_array_append_val(props_array, param);
+	} else if (g_type_is_a(GTK_TYPE_WIDGET, G_PARAM_SPEC_VALUE_TYPE(pspec)) ||
+		   g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(pspec), GTK_TYPE_WIDGET)) {
+	    /* if the pspec could hold a widget, then try to handle it
+	     * later */
+	    deferred_props = g_list_prepend(deferred_props,
+					    &info->properties[i]);
 	}
     }
     widget = g_object_newv(widget_type, props_array->len,
